<?php

namespace TypechoPlugin\SimplePicView;

use Typecho\Plugin\PluginInterface;
use Typecho\Widget\Helper\Form;

if (!defined('__TYPECHO_ROOT_DIR__')) {
    exit;
}

/**
 * 一个简单的图片预览插件，支持全屏预览。只对默认主题进行了适配。有问题可看链接自定义修改
 *
 * @package SimplePicView
 * @author Evlan
 * @version 1.0.0
 * @link https://evlan.cc/archives/typecho-plugins-pic-view.html
 */
class Plugin implements PluginInterface
{
    /**
     * 激活插件方法,如果激活失败,直接抛出异常
     */
    public static function activate()
    {
        \Typecho\Plugin::factory(\Widget\Archive::class)->header = [self::class, 'setHeader'];
        \Typecho\Plugin::factory(\Widget\Archive::class)->footer = [self::class, 'setFooter'];
    }

    public static function setHeader(): void
    {
        echo <<<'EOL'
    <style>
        img {
            max-width: 100%;
            cursor: pointer;
            transition: transform 0.1s ease;
        }

        /* Fullscreen modal with black overlay */
        .fullscreen-overlay {
            position: fixed;
            top: 0;
            left: 0;
            width: 100vw;
            height: 100vh;
            background: rgba(0, 0, 0, 0.8);
            display: flex;
            justify-content: center;
            align-items: center;
            z-index: 1000;
            visibility: hidden;
            opacity: 0;
            transition: opacity 0.3s ease;
        }

        /* When active, overlay is visible */
        .fullscreen-overlay.active {
            visibility: visible;
            opacity: 1;
        }

        /* Disable scrolling */
        body.no-scroll {
            overflow: hidden;
        }

        /* Fullscreen image styling */
        .fullscreen-overlay img {
            max-width: 100%;
            max-height: 100%;
            cursor: grab;
            transform: scale(1) translate(0, 0);
            transition: transform 0.1s ease;
            display: none; /* Initially hide the image until it's loaded */
        }

        .fullscreen-overlay img:active {
            cursor: grabbing;
        }

        /* Loading spinner or text */
        .loading {
            color: white;
            font-size: 24px;
            position: absolute;
            top: 50%;
            left: 50%;
            transform: translate(-50%, -50%);
            display: none;
        }

        /* Show loading when image is not ready */
        .fullscreen-overlay.loading-active .loading {
            display: block;
        }
    </style>
EOL;

    }

    public static function setFooter(): void
    {
        echo <<<'EOL'
<div class="fullscreen-overlay" id="overlay">
    <div class="loading">Loading...</div> <!-- Loading text -->
    <img src="" alt="Fullscreen Image" id="fullscreenImage">
</div>

<script>
    const postContent = document.querySelector('.post-content');
    const overlay = document.getElementById('overlay');
    const fullscreenImage = document.getElementById('fullscreenImage');
    const loadingText = overlay.querySelector('.loading');
    let scale = 1;
    let isDragging = false;
    let startX, startY, currentX = 0, currentY = 0;
    let cachedImages = {}; // Cache for storing loaded images

    // Add event listeners to all images within the .post-content div
    const images = postContent.querySelectorAll('img');
    images.forEach(image => {
        image.addEventListener('click', () => {
            const imgSrc = image.src;

            // Display loading text
            overlay.classList.add('loading-active');
            loadingText.style.display = 'block';

            // Check if image is already cached
            if (!cachedImages[imgSrc]) {
                // Cache the image if it's not cached yet
                fullscreenImage.src = imgSrc;
                cachedImages[imgSrc] = true;

                // Show overlay but keep image hidden until it's fully loaded
                overlay.classList.add('active');
                document.body.classList.add('no-scroll');

                // When the image loads, hide the loading text and display the image
                fullscreenImage.onload = () => {
                    overlay.classList.remove('loading-active');
                    loadingText.style.display = 'none';
                    fullscreenImage.style.display = 'block';
                    resetImageTransform();
                };
            } else {
                // If image is cached, show it immediately
                overlay.classList.add('active');
                document.body.classList.add('no-scroll');
                overlay.classList.remove('loading-active');
                loadingText.style.display = 'none';
                fullscreenImage.style.display = 'block';
                fullscreenImage.src = imgSrc;
                resetImageTransform();
            }
        });
    });

    // Hide fullscreen when clicking on overlay (outside the image)
    overlay.addEventListener('click', (event) => {
        if (event.target === overlay) {
            overlay.classList.remove('active');
            scale = 1;
            fullscreenImage.style.transform = `scale(${scale}) translate(0, 0)`;
            fullscreenImage.style.display = 'none'; // Hide image when closing
            // Re-enable body scrolling
            document.body.classList.remove('no-scroll');
        }
    });

    // Reset the image scale and position when opened
    function resetImageTransform() {
        scale = 1;
        currentX = 0;
        currentY = 0;
        fullscreenImage.style.transform = `scale(${scale}) translate(0, 0)`;
    }

    // Zoom in/out on mouse wheel
    overlay.addEventListener('wheel', (event) => {
        event.preventDefault();
        const zoomSpeed = 0.1;

        if (event.deltaY < 0) {
            // Zoom in
            scale += zoomSpeed;
        } else {
            // Zoom out
            scale = Math.max(1, scale - zoomSpeed);  // Prevent zooming out beyond original size
        }

        fullscreenImage.style.transform = `scale(${scale}) translate(${currentX}px, ${currentY}px)`;
    });

    // Dragging logic
    fullscreenImage.addEventListener('mousedown', (event) => {
        isDragging = true;
        startX = event.clientX - currentX;
        startY = event.clientY - currentY;
        fullscreenImage.style.cursor = 'grabbing';
    });

    document.addEventListener('mousemove', (event) => {
        if (isDragging) {
            currentX = event.clientX - startX;
            currentY = event.clientY - startY;
            fullscreenImage.style.transform = `scale(${scale}) translate(${currentX}px, ${currentY}px)`;
        }
    });

    document.addEventListener('mouseup', () => {
        isDragging = false;
        fullscreenImage.style.cursor = 'grab';
    });

    // Prevent text/image selection while dragging
    fullscreenImage.addEventListener('dragstart', (event) => {
        event.preventDefault();
    });
</script>
EOL;
    }



    /**
     * 禁用插件方法,如果禁用失败,直接抛出异常
     */
    public static function deactivate()
    {
    }

    /**
     * 获取插件配置面板
     *
     * @param Form $form 配置面板
     */
    public static function config(Form $form)
    {
    }

    /**
     * 个人用户的配置面板
     *
     * @param Form $form
     */
    public static function personalConfig(Form $form)
    {
    }

}
